{% extends 'errors/base.html' %}

{% block title %}{{status}}{% endblock %}

{% block body %}
    <h1>{{status}}</h1>
    <p>
        I'll bet you weren't expected that, now were you?
    </p>
{% endblock %}
