{% extends 'errors/base.html' %}

{% block title %}This is a catch all{% endblock %}

{% block body %}
    <h1>Looking for something?</h1>
    <p>
        Hmm, it's not here...
    </p>
{% endblock %}
