<!DOCTYPE html>
<html>
    <head>
        {% block head %}
            <title>{% block title%}Error{% endblock %}</title>
            <style>
                body {
                    margin: 15px;
                    color: #404040;
                    font-family: 'Trebuchet MS', Helvetica, sans-serif;
                    font-size: 12pt;
                }

                h1, h2, h3, h4, h5 {
                    font-weight: bold;
                }

                h1 { font-size: 24pt; }
                h2 { font-size: 21pt; }
                h3 { font-size: 18pt; }
                h4 { font-size: 15pt; }
                h5 { font-size: 12pt; }

                a {
                    color: rgb(87, 139, 218);
                    text-decoration: none;
                }

                pre {
                    margin-left: 30px;
                    font-size: 10pt;
                    color: red;
                    font-family: 'Lucida Console', Monaco, monospace;
                    color: rgb(233, 96, 96);
                }
            </style>
        {% endblock head %}
    </head>
    <body>
        {% block body %}{% endblock %}
    </body>
</html>
