{% extends 'errors/base.html' %}

{% block title %}500 Internal Server Error{% endblock %}


{% block body %}
    <h1>500 Internal Server Error</h1>

    <p>
      {{message}}
    </p>

    <pre>
    {% for line in traceback -%}
    {{line}}
    {%- endfor %}
    </pre>
{% endblock %}
